#pragma once

#include "glew.h"

#include <qwindow.h>
#include <QtGui>

#include <QGLWidget>
#include <QMouseEvent>
#include <iostream>
#include <vector>

#include <glm.hpp>
#include <gtc/matrix_transform.hpp>
#include <gtx/rotate_vector.hpp>

#include "GLWrapper/FBO.h"
#include "GLWrapper/Shader.h"
#include "scene/graph/geometry/Mesh.h"
#include "GLWrapper\VAO.h"
#include "GLWrapper\VBO.h"
#include "GLWrapper\SSBO.h"
#include "GLWrapper\Shader.h"


#include "Grid.h"

class GLWidget : public QGLWidget
{
private:
	std::vector<glm::vec4> atm_list;
	std::vector<std::string> atm_elements;
	std::vector<float> atm_radii;

	bool _init;

	Shader* _shader;
	Shader* _computeShader;

	VAO* _vao;
	SSBO* _ssbo_atm_list;
	SSBO* _ssbo_atm_radii;

	int _windowWidth, _windowHeight;

	Grid* _grid;
	Texture2D* _texture;

	bool _rotating;
	int _oldX;
	int _oldY;
	glm::vec3 _origin;
	glm::vec2 _angles;
	glm::mat4 _rotationMatrix;
	

	glm::vec3 get_arcball_vector(int x, int y);

	void mouseMoveEvent(QMouseEvent *eventMove);
	void mousePressEvent(QMouseEvent *eventPress);
	void mouseReleaseEvent(QMouseEvent *releaseEvent);

protected:
	void initializeGL() Q_DECL_OVERRIDE;
	void paintGL() Q_DECL_OVERRIDE;
	void resizeGL(int width, int height) Q_DECL_OVERRIDE;

public:
	GLWidget(QGLFormat glFormat, QWidget *parent = 0);
	virtual ~GLWidget();

	public slots:
	void cleanup();

	QSize minimumSizeHint() const Q_DECL_OVERRIDE;
	QSize sizeHint() const Q_DECL_OVERRIDE;

	void setPDBData(std::vector<glm::vec4> atoms, std::vector<std::string> elements, std::vector<float> radii);

};

