#include "FBO.h"

FBO::FBO()
{
	glGenFramebuffers(1, &_handle);
	_drawBuffers = nullptr;
}

FBO::~FBO()
{
	glDeleteFramebuffers(1, &_handle);
	if (_drawBuffers != nullptr) delete[] _drawBuffers;
}

bool FBO::setBuffers(std::initializer_list<const Texture2D*> colorBuffers, bool depthRenderBuffer, const Texture2D* depthBufferTex)
{
	if (colorBuffers.size() == 0 && depthBufferTex == nullptr) return false;
	if (colorBuffers.size() != 0) {
		(*(colorBuffers.begin()))->getTextureDimensions(_width, _height);
	} else {
		depthBufferTex->getTextureDimensions(_width, _height);
	}

	bind();

	if (depthRenderBuffer) {
		glGenRenderbuffers(1, &_renderbuffer);
		glBindRenderbuffer(GL_RENDERBUFFER, _renderbuffer);
		glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT, _width, _height);
		glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, _renderbuffer);
	}
	else {
		// depth attachment
		if (depthBufferTex != nullptr) {
			glFramebufferTexture2D(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_TEXTURE_2D, depthBufferTex->getHandle(), 0);
		}
	}

	_nbDrawBuffers = (unsigned int)colorBuffers.size();
	_drawBuffers = new GLenum[_nbDrawBuffers];
	int i = 0;
	for (const Texture2D* tex : colorBuffers) {
		glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0 + i, GL_TEXTURE_2D, tex->getHandle(), 0);
		_drawBuffers[i] = GL_COLOR_ATTACHMENT0 + i;
		++i;
	}

	glDrawBuffers(_nbDrawBuffers, _drawBuffers);

	GLenum status =
		glCheckFramebufferStatus(GL_FRAMEBUFFER);
	if (status != GL_FRAMEBUFFER_COMPLETE)
	{
		return false;
	}

	unbind();

	return true;
}

void FBO::bind(FRAMEBUFFER_BIND_MODE mode) const
{
	glBindFramebuffer((unsigned int)mode, _handle);
	if (_width > 0 && _height > 0) {
		glViewport(0, 0, _width, _height);
	}
}

void FBO::unbind() const
{
	glBindFramebuffer(GL_FRAMEBUFFER, 0);
}