#include "SSBO.h"


SSBO::SSBO(BUFFER_TYPE type, size_t size, glm::vec4* data) : VBO(type), _size(size)
{
	glBindBuffer(target, _handle);
	glBufferData(target, size * sizeof(glm::vec4), data, GL_STATIC_DRAW);
	unbind();
}

SSBO::SSBO(BUFFER_TYPE type, size_t size, float* data) : VBO(type), _size(size)
{
	glBindBuffer(target, _handle);
	glBufferData(target, size * sizeof(float), data, GL_STATIC_DRAW);
	unbind();
}

SSBO::~SSBO()
{
	glDeleteBuffers(1, &_handle);
}

void SSBO::unbind() {
	glBindBuffer(target, 0);
}

void SSBO::bindBase(int index) {
	glBindBufferBase(target, index,  _handle);
}

void SSBO::unbindBase(int index) {
	glBindBufferBase(target, index, 0);
}

glm::vec4* SSBO::map(GLbitfield access)
{
	bind();
	return (glm::vec4*)glMapBufferRange(target, 0, _size*sizeof(glm::vec4), access);
}

GLuint* SSBO::mapGLuint(GLbitfield access)
{
	bind();
	return (GLuint*)glMapBufferRange(target, 0, _size * sizeof(GLuint), access);
}

void SSBO::unmap()
{
	bind();
	glUnmapBuffer(target);
}

void SSBO::bindToVAO()
{
	int index = _type->index;
	glBindBuffer(GL_ARRAY_BUFFER, _handle);
	glEnableVertexAttribArray(index);
	glVertexAttribPointer(index, 4, GL_FLOAT, GL_FALSE, 0, 0);
	glVertexAttribDivisor(index, 1); // set to 1, so for each instance one vec4 is taken
}