#pragma once

#include <string>
#include <fstream>
#include <iostream>
#include <glew.h>
#include <glm.hpp>
#include <gtc/type_ptr.hpp>

#include "Texture2D.h"
#include "CubeMapTexture.h"
#include "Texture3D.h"
#include "../scene/graph/Light.h"

#define EXT_VERTEX_SHADER ".vert"
#define EXT_FRAGMENT_SHADER ".frag"
#define EXT_GEOMETRY_SHADER ".geom"
#define EXT_TESSELATION_CONTROL_SHADER ".tesc"
#define EXT_TESSELATION_EVALUATION_SHADER ".tese"
#define EXT_COMPUTE_SHADER ".comp"

class Shader
{
protected:
	GLuint _handle;
	std::string _file;
	bool _computeShader;

	GLuint loadShaders(bool computeShader);
	bool loadShader_(std::string file, GLenum shaderType, std::string ext, GLuint& handle);

public:

	Shader(std::string file, bool computeShader = false); // file path w/o extension!
	virtual ~Shader();

	void bind();
	void unbind();

	void reload();

	void setUniform(const char* name, const int i);
	void setUniform(GLint location, const int i);
	void setUniform(const char* name, const unsigned int i);
	void setUniform(GLint location, const unsigned int i);
	void setUniform(const char* name, const float f);
	void setUniform(GLint location, const float f);
	void setUniform(const char* name, const glm::mat4& mat);
	void setUniform(GLint location, const glm::mat4& mat);
	void setUniform(const char* name, const glm::mat3& mat);
	void setUniform(GLint location, const glm::mat3& mat);
	void setUniform(const char* name, const glm::vec2& vec);
	void setUniform(GLint location, const glm::vec2& vec);
	void setUniform(const char* name, const glm::vec3& vec);
	void setUniform(GLint location, const glm::vec3& vec);
	void setUniform(const char* name, const glm::vec4& vec);
	void setUniform(GLint location, const glm::vec4& vec);
	void setUniform(const char* name, const Texture2D* tex, unsigned int unit);
	void setUniform(const char* name, const CubeMapTexture* tex, unsigned int unit);
	void setUniform(const char* name, const Texture3D* tex, unsigned int unit);
	void setUniform(const char* name, Light* l);
	void setUniformArr(const char* name, Light* l, int index);
	void setUniformArr(const char* name, const glm::vec2* arr, int index);
	void setUniform(const char* name, const glm::mat4x2& mat);
	void setUniform(GLint location, const glm::mat4x2& mat);
};

