#pragma once

#include "GLTypes.h"

class Texture2D
{
private:

	GLuint _handle;
	int _width, _height;
	bool _srgb;
	TEXTURE_FORMAT _format;

	void initTex2D(TEXTURE_FORMAT format, TEXTURE_COMPONENTS components, TEXTURE_DATA_TYPE datatype, SAMPLING_MODE sampling, SAMPLER_WRAP_MODE wrap, const void* data);

public:

	Texture2D(unsigned int width, unsigned int height, TEXTURE_FORMAT format, TEXTURE_COMPONENTS components, TEXTURE_DATA_TYPE datatype, SAMPLING_MODE sampling = SAMPLING_MODE::TRILINEAR, SAMPLER_WRAP_MODE wrap = SAMPLER_WRAP_MODE::CLAMP_TO_BORDER, bool srgb = false);
	Texture2D(const void* data, unsigned int width, unsigned int height, TEXTURE_FORMAT format, TEXTURE_COMPONENTS components, TEXTURE_DATA_TYPE datatype, SAMPLING_MODE sampling = SAMPLING_MODE::TRILINEAR, SAMPLER_WRAP_MODE wrap = SAMPLER_WRAP_MODE::CLAMP_TO_BORDER, bool srgb = false);
	~Texture2D();

	void getTextureDimensions(int& width, int& height) const;
	void bind(unsigned int unit) const;
	void bindImage(unsigned int unit) const;
	void unbind() const;
	unsigned int getHandle() const;
	void generateMipMaps() const;
	void setSamplingMode(SAMPLING_MODE samplingmode);
	bool isSRGBImage() const;
};

