#include "Texture3D.h"

void Texture3D::initTex3D(TEXTURE_FORMAT format, TEXTURE_COMPONENTS components, TEXTURE_DATA_TYPE datatype, SAMPLING_MODE sampling, SAMPLER_WRAP_MODE wrap, const void* data)
{
	_format = format;
	glGenTextures(1, &_handle);

	glBindTexture(_texTarget, _handle);
	glTexImage3D(_texTarget, 0, (unsigned int)format, _width, _height, _depth, 0, (unsigned int)components, (unsigned int)datatype, data);


	glTexParameteri(_texTarget, GL_TEXTURE_WRAP_S, (unsigned int)wrap);
	glTexParameteri(_texTarget, GL_TEXTURE_WRAP_T, (unsigned int)wrap);
	glTexParameteri(_texTarget, GL_TEXTURE_WRAP_R, (unsigned int)wrap);
	switch (sampling)
	{
	case SAMPLING_MODE::LINEAR:
		glTexParameteri(_texTarget, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(_texTarget, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		break;
	case SAMPLING_MODE::BILINEAR:
		glTexParameteri(_texTarget, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(_texTarget, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		break;
	case SAMPLING_MODE::TRILINEAR:
		glTexParameteri(_texTarget, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(_texTarget, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		break;
	}

	if (data != nullptr) {
		glGenerateMipmap(_texTarget);
	}

	unbind();
}

Texture3D::Texture3D(unsigned int width, unsigned int height, unsigned int depth, TEXTURE_FORMAT format, TEXTURE_COMPONENTS components, TEXTURE_DATA_TYPE datatype, SAMPLING_MODE sampling, SAMPLER_WRAP_MODE wrap, bool multisampling)
	: Texture3D(nullptr, width, height, depth, format, components, datatype, sampling, wrap, multisampling)
{
}

Texture3D::Texture3D(const void* data, unsigned int width, unsigned int height, unsigned int depth, TEXTURE_FORMAT format, TEXTURE_COMPONENTS components, TEXTURE_DATA_TYPE datatype, SAMPLING_MODE sampling, SAMPLER_WRAP_MODE wrap, bool multisampling)
	: _width(width), _height(height), _depth(depth), _texTarget(GL_TEXTURE_3D)
{
	initTex3D(format, components, datatype, sampling, wrap, data);
}

Texture3D::~Texture3D()
{
	glDeleteTextures(1, &_handle);
}

void Texture3D::getTextureDimensions(int& width, int& height, int& depth) const
{
	width = _width;
	height = _height;
	depth = _depth;
}

void Texture3D::bind(unsigned int unit) const
{
	glActiveTexture(GL_TEXTURE0 + unit);
	glBindTexture(_texTarget, _handle);
}

void Texture3D::bindImage(unsigned int unit) const
{
	glActiveTexture(GL_TEXTURE0 + unit);
	GLenum texTarget = GL_TEXTURE_3D;
	glBindTexture(texTarget, _handle);
	glBindImageTexture(unit, _handle, 0, GL_TRUE, 0, GL_READ_WRITE, (unsigned int)_format);
}

void Texture3D::unbind() const
{
	GLenum texTarget = GL_TEXTURE_3D;

	glBindTexture(texTarget, 0);
}

unsigned int Texture3D::getHandle() const
{
	return _handle;
}

void Texture3D::generateMipMaps() const
{
	glBindTexture(_texTarget, _handle);
	glGenerateMipmap(_texTarget);
	glBindTexture(_texTarget, 0);
}