#include "VAO.h"


VAO::VAO()
{
	glGenVertexArrays(1, &_handle);
}

VAO::~VAO()
{
	glDeleteVertexArrays(1, &_handle);
}

void VAO::bind()
{
	glBindVertexArray(_handle);
}

void VAO::unbind()
{
	glBindVertexArray(0);
}

void VAO::setBuffers(std::initializer_list<VBO*> buffers)
{
	setBuffers(nullptr, buffers);
}

void VAO::setBuffers(IBO* indices, std::initializer_list<VBO*> buffers)
{
	bind();
	for (const VBO* vbo : buffers) {
		if (vbo != nullptr) {
			vbo->bindToVAO();
		}
	}
	if(indices != nullptr) indices->bindToVAO();
	unbind();
}

void VAO::setBuffers(std::vector<VBO*> buffers)
{
	setBuffers(nullptr, buffers);
}

void VAO::setBuffers(IBO* indices, std::vector<VBO*> buffers)
{
	bind();
	for (const VBO* vbo : buffers) {
		if (vbo != nullptr) {
			vbo->bindToVAO();
		}
	}
	if (indices != nullptr) indices->bindToVAO();
	unbind();
}