#include "VBO.h"


static BUFFER_TYPE_DATA _BUFFER_DATA[] = {
	{ GL_ARRAY_BUFFER, GL_STATIC_DRAW, POSITIONS_BUFFER_LOCATION, GL_FLOAT, sizeof(float), 3 },
	{ GL_ARRAY_BUFFER, GL_STATIC_DRAW, NORMALS_BUFFER_LOCATION, GL_FLOAT, sizeof(float), 3 },
	{ GL_ARRAY_BUFFER, GL_STATIC_DRAW, COLORS_BUFFER_LOCATION, GL_FLOAT, sizeof(float), 3 },
	{ GL_ARRAY_BUFFER, GL_STATIC_DRAW, UVS1_BUFFER_LOCATION, GL_FLOAT, sizeof(float), 2 },
	{ GL_ARRAY_BUFFER, GL_STATIC_DRAW, UVS2_BUFFER_LOCATION, GL_FLOAT, sizeof(float), 2 },
	{ GL_ARRAY_BUFFER, GL_STATIC_DRAW, TANGENTS_BUFFER_LOCATION, GL_FLOAT, sizeof(float), 3 },
	{ GL_ARRAY_BUFFER, GL_STATIC_DRAW, BITANGENTS_BUFFER_LOCATION, GL_FLOAT, sizeof(float), 3 },
	{ GL_ARRAY_BUFFER, GL_STATIC_DRAW, BONE_WEIGHTS_BUFFER_LOCATION, GL_FLOAT, sizeof(float), 4 },
	{ GL_ARRAY_BUFFER, GL_STATIC_DRAW, BONE_INDICES_BUFFER_LOCATION, GL_FLOAT, sizeof(float), 4 },
	{ GL_ARRAY_BUFFER, GL_STATIC_DRAW, SCALAR_BUFFER_LOCATION, GL_FLOAT, sizeof(float), 1 }
};

VBO::VBO(BUFFER_TYPE type)
{
	_type = &_BUFFER_DATA[(int)type];

	glGenBuffers(1, &_handle);
}

VBO::VBO(BUFFER_TYPE type, int vertices) : VBO(type, vertices, nullptr)
{
}

VBO::VBO(BUFFER_TYPE type, int vertices, void* data) 
{	
	_vertices = vertices;
	_type = &_BUFFER_DATA[(int)type];	
	
	glGenBuffers(1, &_handle);
	glBindBuffer(_type->type, _handle);
	glBufferData(_type->type, _type->typeSize * _type->components * vertices, data, _type->mode);
}

VBO::~VBO()
{
	glDeleteBuffers(1, &_handle);
}

void VBO::bind() const
{
	glBindBuffer(_type->type, _handle);
}

void VBO::bindToVAO() const
{
	glBindBuffer(_type->type, _handle);
	glEnableVertexAttribArray(_type->index);
	glVertexAttribPointer(_type->index, _type->components, _type->dataType, GL_FALSE, 0, 0);
}

void VBO::updateData(void* data)
{
	glBindBuffer(_type->type, _handle);
	glBufferData(_type->type, _type->typeSize * _type->components * _vertices, data, _type->mode);	
}

unsigned int VBO::getLocation() const 
{
	return _type->index;
}