#pragma once

#include "GLTypes.h"


#define VBO_MAX_ATTRIBUTES 9

#define POSITIONS_BUFFER_LOCATION 0
#define NORMALS_BUFFER_LOCATION 1
#define COLORS_BUFFER_LOCATION 2
#define UVS1_BUFFER_LOCATION 3
#define UVS2_BUFFER_LOCATION 4
#define TANGENTS_BUFFER_LOCATION 5
#define BITANGENTS_BUFFER_LOCATION 6
#define BONE_WEIGHTS_BUFFER_LOCATION 7
#define BONE_INDICES_BUFFER_LOCATION 8
#define SCALAR_BUFFER_LOCATION 9

struct BUFFER_TYPE_DATA {
	GLenum type;
	GLenum mode;
	GLuint index;
	GLenum dataType;
	GLint typeSize;
	GLint components;
};


class VBO
{
protected:

	GLuint _handle;
	BUFFER_TYPE_DATA* _type;

	int _vertices;

public:

	enum class BUFFER_TYPE {
		POSITIONS = 0, NORMALS = 1, COLORS = 2, UVS1 = 3, UVS2 = 4, TANGENTS = 5, BITANGENTS = 6, BONE_WEIGHTS = 7, BONE_INDICES = 8, SCALAR = 9
	};

	// size is number of vertices, size of buffer is: size * dataType * components
	VBO(BUFFER_TYPE type);
	VBO(BUFFER_TYPE type, int vertices);
	VBO(BUFFER_TYPE type, int vertices, void* data);
	~VBO();

	void bind() const;
	void bindToVAO() const;
	void updateData(void* data);
	unsigned int getLocation() const;
};

