#include "Grid.h"

const int Grid::MAX_CELL_PRE_ENTRIES = 1;
const int Grid::MAX_CELL_ENTRIES = 27;

Grid::Grid(std::vector<glm::vec4> atoms, float maximumSolventRadius, float maximumAtomRadius)
{
	gridCellSize = 2.0f * maximumAtomRadius + 2.0f * maximumSolventRadius;
	_ssbo_grid_data = nullptr;

	initialize(atoms);
}

void Grid::initialize(std::vector<glm::vec4> atoms) {
	// calculate min und max of atoms
	glm::vec3 max(-FLT_MAX);
	glm::vec3 min(FLT_MAX);
	for (size_t i = 0; i < atoms.size(); i++) {
		if (atoms[i].x > max.x) {
			max.x = atoms[i].x;
		}
		if (atoms[i].y > max.y) {
			max.y = atoms[i].y;
		}
		if (atoms[i].z > max.z) {
			max.z = atoms[i].z;
		}

		if (atoms[i].x < min.x) {
			min.x = atoms[i].x;
		}
		if (atoms[i].y < min.y) {
			min.y = atoms[i].y;
		}
		if (atoms[i].z < min.z) {
			min.z = atoms[i].z;
		}
	}

	_gridSize = ceil((max + gridCellSize - min + gridCellSize) / gridCellSize);

	std::vector<glm::vec4[MAX_CELL_PRE_ENTRIES]> gridData = std::vector<glm::vec4[MAX_CELL_PRE_ENTRIES]>(static_cast<size_t>(_gridSize.x * _gridSize.y * _gridSize.z));
	for (size_t i = 0; i < gridData.size(); i++) {
		gridData[i][0] = glm::vec4(0,0,0,-1);
	}

	for (size_t i = 0; i < atoms.size(); i++) {
		glm::vec3 gridIndex = floor((glm::vec3(atoms[i]) - min + gridCellSize) / gridCellSize);
		size_t arrIndex = static_cast<size_t>(gridIndex.x + gridIndex.y * _gridSize.x + gridIndex.z * _gridSize.x * _gridSize.y);

		size_t cellIndex = 0;
		int atomIndex = static_cast<int>(gridData[arrIndex][cellIndex].w);
		while (atomIndex != -1 && cellIndex < MAX_CELL_PRE_ENTRIES) {
			cellIndex++;
			atomIndex = static_cast<int>(gridData[arrIndex][cellIndex].w);
		}
		if (cellIndex == MAX_CELL_PRE_ENTRIES) {
			//std::cout << "Cell " << arrIndex << " has more than " << MAX_CELL_PRE_ENTRIES << " (limit) entries."  << std::endl;
			continue;
		}

		gridData[arrIndex][cellIndex] = atoms[i];
		if (cellIndex < MAX_CELL_PRE_ENTRIES - 1) {
			gridData[arrIndex][cellIndex+1] = glm::vec4(0, 0, 0, -1);
		}
	} 

	std::vector<glm::vec4[MAX_CELL_ENTRIES]> gridDataPreProcessed = std::vector<glm::vec4[MAX_CELL_ENTRIES]>(static_cast<size_t>(_gridSize.x * _gridSize.y * _gridSize.z));
	for (size_t i = 0; i < gridData.size(); i++) {
		gridDataPreProcessed[i][0] = glm::vec4(0, 0, 0, -1);
	}

	for (unsigned int x = 0; x < _gridSize.x; x++) {
		for (unsigned int y = 0; y < _gridSize.y; y++) {
			for (unsigned int z = 0; z < _gridSize.z; z++) {

				size_t cellIndex = 0;
				size_t gridIndex = static_cast<size_t>((x)+(y)* _gridSize.x + (z)* _gridSize.x * _gridSize.y);

				for (int i = -1; i < 2; i++) {
					for (int j = -1; j < 2; j++) {
						for (int k = -1; k < 2; k++) {
							if (x + i >= 0 && x + i < _gridSize.x
								&& y + j >= 0 && y + j < _gridSize.y
								&& z + k >= 0 && z + k < _gridSize.z) {
								size_t arrIndex = static_cast<size_t>((x + i) + (y + j) * _gridSize.x + (z + k) * _gridSize.x * _gridSize.y);
								size_t localIndex = 0;

								int atomIndex = static_cast<int>(gridData[arrIndex][localIndex].w);
								while (atomIndex != -1 && cellIndex < MAX_CELL_ENTRIES && localIndex < MAX_CELL_PRE_ENTRIES) {
									gridDataPreProcessed[gridIndex][cellIndex] = gridData[arrIndex][localIndex];
									cellIndex++;
									localIndex++;
									atomIndex = static_cast<int>(gridData[arrIndex][localIndex].w);
								}
								if (cellIndex == MAX_CELL_ENTRIES) {
									//std::cout << "Preprocess: Cell " << gridIndex << " has more than " << MAX_CELL_ENTRIES << " (limit) entries." << std::endl;
									continue;
								}
							}

						}
					}
				}
			}
		}
	}



	_ssbo_grid_data = new SSBO(VBO::BUFFER_TYPE::POSITIONS, gridData.size() * MAX_CELL_ENTRIES, &gridDataPreProcessed[0][0]);
}


Grid::~Grid()
{
	if (_ssbo_grid_data != nullptr) delete _ssbo_grid_data;
}


