#include "MainWindow.h"

#include <QFileDialog>

#include <QPainter>

#include <QtGui>
#include <QHBoxLayout>

MainWindow::MainWindow(QWidget *parent)
	: QMainWindow(parent)
{
	QSurfaceFormat format;
	format.setRenderableType(QSurfaceFormat::OpenGL);
	format.setProfile(QSurfaceFormat::CoreProfile);
	format.setVersion(4, 3);

#ifdef    _DEBUG
	format.setOption(QSurfaceFormat::DebugContext);
#endif // _DEBUG

	

	QGLFormat glFormat;
	glFormat.fromSurfaceFormat(format);
	//glFormat.setVersion(4, 3);
	//glFormat.setProfile(QGLFormat::CoreProfile); // Requires >=Qt-4.8.0
	//glFormat.setSampleBuffers(true);

	_glwidget = new GLWidget(glFormat);
	m_Ui = new Ui_MainWindow();
	m_Ui->setupUi(this);

	QVBoxLayout* glWidgetLayout = findChild<QVBoxLayout*>("glWidgetLayout");
	glWidgetLayout->addWidget(_glwidget);

	QTimer *timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(updateGLWidget()));
	timer->start(1000.0f/60.0f); // 60 fps

	connect(m_Ui->actionOpen, SIGNAL(triggered()), this, SLOT(openFileAction()));
	connect(m_Ui->actionClose, SIGNAL(triggered()), this, SLOT(closeAction()));

	// scale, rotation, translation slider
	/*connect(m_Ui->scaleSlider, SIGNAL(valueChanged(int)), this, SLOT(setScale(int)));
	connect(m_Ui->xRotSlider, SIGNAL(valueChanged(int)), this, SLOT(setRotationX(int)));
	connect(m_Ui->yRotSlider, SIGNAL(valueChanged(int)), this, SLOT(setRotationY(int)));
	connect(m_Ui->zRotSlider, SIGNAL(valueChanged(int)), this, SLOT(setRotationZ(int)));
	connect(m_Ui->xTransSlider, SIGNAL(valueChanged(int)), this, SLOT(setTransX(int)));
	connect(m_Ui->yTransSlider, SIGNAL(valueChanged(int)), this, SLOT(setTransY(int)));
	connect(m_Ui->zTransSlider, SIGNAL(valueChanged(int)), this, SLOT(setTransZ(int)));
	connect(m_Ui->thresholdSlider, SIGNAL(valueChanged(int)), this, SLOT(setThreshold(int)));
	connect(m_Ui->minAlphaSlider, SIGNAL(valueChanged(int)), this, SLOT(setMinAlpha(int)));

	connect(m_Ui->mipRB, SIGNAL(toggled(bool)), this, SLOT(setCurVolumeRendererMIP(bool)));
	connect(m_Ui->mipcpuRB, SIGNAL(toggled(bool)), this, SLOT(setCurVolumeRendererMIPCPU(bool)));
	connect(m_Ui->gradientRB, SIGNAL(toggled(bool)), this, SLOT(setCurVolumeRendererGradient(bool)));
	connect(m_Ui->alphaCompRB, SIGNAL(toggled(bool)), this, SLOT(setCurVolumeRendererAlphaComp(bool)));
	connect(m_Ui->avgRB, SIGNAL(toggled(bool)), this, SLOT(setCurVolumeRendererAvg(bool)));*/

	// reset btn
	//connect(m_Ui->resetBtn, SIGNAL(pressed()), this, SLOT(reset()));
}

MainWindow::~MainWindow()
{
}


//-------------------------------------------------------------------------------------------------
// Slots
//-------------------------------------------------------------------------------------------------

void MainWindow::openFileAction()
{
	QString filename = QFileDialog::getOpenFileName(this, "Data File", 0, tr("Data Files (*.pdb)"));

	if (!filename.isEmpty())
	{
		// store filename
		m_FileType.filename = filename;
		std::string fn = filename.toStdString();
		bool success = false;

		// progress bar and top label
		m_Ui->progressBar->setEnabled(true);
		m_Ui->labelTop->setText("Loading data ...");
				
		// load file
		std::vector<glm::vec4> atoms;
		std::vector<std::string> elements;
		std::vector<float> radii;
		success = _pdbLoader.loadPDB(fn, atoms, elements, radii);

		m_Ui->progressBar->setEnabled(false);

		// status message
		if (success)
		{
			m_Ui->labelTop->setText("File LOADED [" + filename + "] - Type [PDB]");

			// set data in GLWidget
			_glwidget->setPDBData(atoms, elements, radii);
		}
		else
		{
			m_Ui->labelTop->setText("ERROR loading file " + filename + "!");
			m_Ui->progressBar->setValue(0);
		}
	}
}

void MainWindow::closeAction()
{
	close(); 
}

void MainWindow::updateGLWidget() {
	_glwidget->update();
}
