#include "PDBLoader.h"

using namespace std;

PDBLoader::PDBLoader()
{
}

PDBLoader::~PDBLoader()
{
}

bool PDBLoader::loadPDB(std::string file, std::vector<glm::vec4>& atom_list, std::vector<std::string>& elements, std::vector<float>& radii)
{
	ESBTL::PDB_line_selector_two_systems sel;

	vector<ESBTL::Default_system> systems;
	ESBTL::All_atom_system_builder<ESBTL::Default_system> builder(systems, sel.max_nb_systems());
	
	T_Atom_classifier atom_classifier;

	if (ESBTL::read_a_pdb_file(file, sel, builder, Accept_none_occupancy_policy())){
		if (systems.empty() || systems[0].has_no_model()){
			return false;
		}
		vector<string> t_elements;
		vector<float> t_radii;
		//Consider only the first model of the first system
		const ESBTL::Default_system::Model& model = *systems[0].models_begin();
		for (ESBTL::Default_system::Model::Atoms_const_iterator it_atm = model.atoms_begin(); it_atm != model.atoms_end(); ++it_atm){
			atom_list.push_back(glm::vec4(it_atm->x(), it_atm->y(), it_atm->z(), 1));
			t_elements.push_back(it_atm->element());
			t_radii.push_back(static_cast<float>(atom_classifier.get_properties(*it_atm).value()));
		}

		for (unsigned i = 0; i < atom_list.size(); i++) {
			int idx_exist = -1;
			for (unsigned j = 0; j < elements.size(); j++) { // search for existing elements
				if (elements[j] == t_elements[i]) { // element already exists
					idx_exist = j;
					break;
				}
			}
			if (idx_exist == -1) { // create new element
				elements.push_back(t_elements[i]);
				radii.push_back(t_radii[i]);
				idx_exist = elements.size() - 1;
			}
			atom_list[i].w = (float)idx_exist; // set id in w-component of atoms position
		}
	}

	return true;
}
