#pragma once

#include <glm.hpp>

#include <vector>


struct AABB {
	glm::vec3 box[8];

	AABB() { }

	AABB(const glm::vec3 box_[8])
	{
		for (int i = 0; i < 8; i++) {
			box[i] = box_[i];
		}
	}

	AABB(const std::vector<glm::vec3>& verts) 
	{
		glm::vec3 minAABB(FLT_MAX);
		glm::vec3 maxAABB(-FLT_MAX);
		for (unsigned int i = 0; i < verts.size(); i++) {
			glm::vec3 v = verts[i];
			if (v.x < minAABB.x) minAABB.x = v.x;
			if (v.y < minAABB.y) minAABB.y = v.y;
			if (v.z < minAABB.z) minAABB.z = v.z;
			if (v.x > maxAABB.x) maxAABB.x = v.x;
			if (v.y > maxAABB.y) maxAABB.y = v.y;
			if (v.z > maxAABB.z) maxAABB.z = v.z;
		}

		initBox(minAABB, maxAABB);
	}

	AABB(const std::vector<AABB>& aabbs)
	{
		glm::vec3 min_glob(FLT_MAX);
		glm::vec3 max_glob(-FLT_MAX);
		for (unsigned int i = 0; i < aabbs.size(); i++) {
			glm::vec3 min_aabb = aabbs[i].minV();
			glm::vec3 max_aabb = aabbs[i].maxV();
			if (min_aabb.x < min_glob.x) min_glob.x = min_aabb.x;
			if (min_aabb.y < min_glob.y) min_glob.y = min_aabb.y;
			if (min_aabb.z < min_glob.z) min_glob.z = min_aabb.z;
			if (max_aabb.x > max_glob.x) max_glob.x = max_aabb.x;
			if (max_aabb.y > max_glob.y) max_glob.y = max_aabb.y;
			if (max_aabb.z > max_glob.z) max_glob.z = max_aabb.z;
		}

		initBox(min_glob, max_glob);
	}

	AABB transform(glm::mat4 M) const {
		AABB a(box);
		for (int i = 0; i < 8; i++) {
			a.box[i] = glm::vec3(M * glm::vec4(a.box[i], 1));
		}
		return a;
	}

	void initBox(glm::vec3 minV, glm::vec3 maxV) {
		box[0] = glm::vec3(minV.x, minV.y, minV.z); // left bottom near
		box[1] = glm::vec3(minV.x, minV.y, maxV.z); // left bottom far
		box[2] = glm::vec3(minV.x, maxV.y, minV.z); // left top near
		box[3] = glm::vec3(minV.x, maxV.y, maxV.z); // left top far
		box[4] = glm::vec3(maxV.x, minV.y, minV.z); // right bottom near
		box[5] = glm::vec3(maxV.x, minV.y, maxV.z); // right bottom far
		box[6] = glm::vec3(maxV.x, maxV.y, minV.z); // right top near
		box[7] = glm::vec3(maxV.x, maxV.y, maxV.z); // right top far
	}

	glm::vec3 minV() const {
		return box[0];
	}

	glm::vec3 maxV() const {
		return box[7];
	}

	glm::vec3 center() const {
		return (box[7] + box[0]) * 0.5f;
	}

	bool inside(glm::vec3 v) const {
		return (minV().x < v.x && minV().y < v.y && minV().z < v.z && maxV().x > v.x && maxV().y > v.y && maxV().z > v.z);
	}

};