#include "Camera.h"


float interpolate(float a, float b, float alpha);

Camera::Camera(glm::mat4 proj) : _proj(proj)
{
	_view = glm::lookAt(glm::vec3(0), glm::vec3(0, 0, -1), glm::vec3(0, 1, 0));

	_horizontalAngle = 0.0f;
	_verticalAngle = 0.0f;
	_debugEnabled = false;
	_debugTranslationSpeed = 1.0f;
}

Camera::~Camera()
{
}

void Camera::lookAt(glm::vec3 pos, glm::vec3 target, glm::vec3 up)
{
	_view = glm::lookAt(pos, target, up);
	_position = pos;
}

void Camera::updateDebug(float dt, int cursor_dx, int cursor_dy) 
{
	if (_debugEnabled) {
		float sensitivity = 0.2f;
		_horizontalAngle += sensitivity * float(cursor_dx);
		_verticalAngle += sensitivity * float(cursor_dy);

		if (_horizontalAngle > 360.0f) _horizontalAngle -= 360.0f;
		else if (_horizontalAngle < 0.0f) _horizontalAngle += 360.0f;

		if (_verticalAngle > 80.0f) _verticalAngle = 80.0f;
		else if (_verticalAngle < -80.0f) _verticalAngle = -80.0f;

		glm::mat4 rotPitch = glm::rotate(glm::mat4(1), glm::radians(_verticalAngle), glm::vec3(1, 0, 0));
		glm::mat4 rotYaw = glm::rotate(glm::mat4(1), glm::radians(_horizontalAngle), glm::vec3(0, 1, 0));

		glm::vec3 forward(_view[0][2], _view[1][2], _view[2][2]);
		glm::vec3 strafe(_view[0][0], _view[1][0], _view[2][0]);
		_position += (-_translation.x * forward + _translation.y * strafe) * 4.0f  * dt;

		_view = (rotPitch * rotYaw) * glm::translate(glm::mat4(1), -_position);
	}
}

void Camera::setDebugTranslationZ(float z) 
{
	_translation.x = z * _debugTranslationSpeed;
}

void Camera::setDebugTranslationX(float x)
{
	_translation.y = x *_debugTranslationSpeed;
}

void Camera::toggleDebugCamera() 
{
	_debugEnabled = !_debugEnabled;
}

bool Camera::inDebugMode()
{
	return _debugEnabled;
}

glm::mat4 Camera::getViewMatrix() const
{
	return _view;
}

glm::mat4 Camera::getProjectionMatrix() const
{
	return _proj;
}

glm::vec3 Camera::getPosition() const
{
	return _position;
}

glm::vec3 Camera::getDirection() const
{
	return -glm::vec3(_view[0][2], _view[1][2], _view[2][2]); // look down the negative z axis
}

glm::vec3 Camera::getUp() const
{
	return glm::vec3(_view[0][1], _view[1][1], _view[2][1]);
}

glm::vec3 Camera::getLeft() const
{
	return glm::vec3(_view[0][0], _view[1][0], _view[2][0]);
}

void Camera::setProjectionMatrix(glm::mat4 proj)
{
	_proj = proj;
}