#pragma once

#include <map>

#include <glm.hpp>
#include <gtc/matrix_transform.hpp>
#include <gtx\string_cast.hpp>


class Camera
{
private:
	glm::mat4 _view;
	glm::mat4 _proj;

	// debug camera
	bool _debugEnabled;
	float _horizontalAngle;
	float _verticalAngle;
	glm::vec2 _translation;
	glm::vec3 _position;
	float _debugTranslationSpeed;

public:
	Camera(glm::mat4 proj);
	~Camera();

	void lookAt(glm::vec3 pos, glm::vec3 target, glm::vec3 up);
	void setProjectionMatrix(glm::mat4 proj);

	void toggleDebugCamera();
	bool inDebugMode();
	void updateDebug(float dt, int cursor_dx, int cursor_dy);
	void setDebugTranslationZ(float z); // 1.0 = forward, -1.0 = backward
	void setDebugTranslationX(float x); // 1.0 = right, -1.0 = left

	glm::mat4 getViewMatrix() const;
	glm::mat4 getProjectionMatrix() const;
	glm::vec3 getPosition() const;
	glm::vec3 getDirection() const;
	glm::vec3 getUp() const;
	glm::vec3 getLeft() const;
};

