#include "SceneGraph.h"


SceneGraph::SceneGraph(SceneObject* root) 
	: _root(root)
{
	_root->initAABB(glm::mat4(1));
}

SceneGraph::~SceneGraph()
{
	delete _root;

	for (unsigned int i = 0; i < _pointLights.size(); i++) {
		delete _pointLights[i];
	}

	for (unsigned int i = 0; i < _directionalLights.size(); i++) {
		delete _directionalLights[i];
	}

	for (unsigned int i = 0; i < _postRenderList.size(); i++) {
		delete _postRenderList[i];
	}
}

void SceneGraph::render(RenderInfo& renderinfo, const Camera* camera) const
{
	glm::mat4 V = camera->getViewMatrix();
	glm::mat4 P = camera->getProjectionMatrix();
	

	_root->render(renderinfo, glm::mat4(1), V, P);

	for (unsigned int i = 0; i < _postRenderList.size(); i++) {
		_postRenderList[i]->render(renderinfo, _root->getModelMatrix(), V, P);
	}
}

void SceneGraph::update(float dt)
{
	_root->update(dt);

	for (unsigned int i = 0; i < _postRenderList.size(); i++) {
		_postRenderList[i]->update(dt);
	}
}

void SceneGraph::addDirectionalLight(Light * light)
{
	_directionalLights.push_back(light);
}

void SceneGraph::addPointLight(Light * light)
{
	_pointLights.push_back(light);
}

void SceneGraph::addPostRenderObject(SceneObject* obj)
{
	_postRenderList.push_back(obj);
}

AABB SceneGraph::getSceneAABB() const
{
	std::vector<AABB> aabbs;
	aabbs.push_back(_root->getAABB());

	return AABB(aabbs);
}

SceneObject* SceneGraph::find(std::string name)
{
	SceneObject* ret = nullptr;
	ret = _root->find(name);

	if (ret != nullptr) return ret;

	for (unsigned int i = 0; i < _postRenderList.size(); i++) {
		ret = _postRenderList[i]->find(name);
		if (ret != nullptr) break;
	}

	return ret;
}
