#pragma once

#include "../../util/Global.h"

#include "Camera.h"
#include "SceneObject.h"
#include "../../render/RenderInfo.h"
#include "Light.h"

#include <gtc\matrix_transform.hpp>
#include <gtx\transform.hpp>

#include <vector>

class SceneGraph
{
private:
	SceneObject* _root;
	std::vector<SceneObject*> _postRenderList;

	std::vector<Light*> _directionalLights;
	std::vector<Light*> _pointLights;

public:

	SceneGraph(SceneObject* root);
	virtual ~SceneGraph();

	void render(RenderInfo& renderinfo, const Camera* camera) const;
	void update(float dt);
	void startAnimations();

	SceneObject* getRoot() { return _root; }
	AABB getSceneAABB() const;
	SceneObject* find(std::string name);

	std::vector<Light*> getDirectionalLights() const { return _directionalLights; }
	std::vector<Light*> getPointLights() const { return _pointLights; }

	void addDirectionalLight(Light* light);
	void addPointLight(Light* light);
	void addPostRenderObject(SceneObject* obj);
};

