#pragma once

#include <glm.hpp>
#include <gtc\matrix_transform.hpp>
#include <queue>
#include <vector>

#include "geometry/Mesh.h"
#include "../../render/RenderInfo.h"


class SceneObject
{
private:
	std::string _name;

	SceneObject* _parent;
	std::vector<SceneObject*> _children;
	std::vector<Mesh*> _meshes;

	bool _enabled;
	
	void addChild(SceneObject* child);
	void initAABBRec(const glm::mat4& Maccum);

protected:
	glm::mat4 _model;
	glm::mat4 _inverseModel;
	
	virtual void postUpdate(float dt);

public:
	SceneObject(std::string name);
	SceneObject(std::string name, const glm::mat4& model);
	SceneObject(std::string name, const glm::mat4& model, SceneObject* parent);
	virtual ~SceneObject();

	glm::mat4 getModelMatrix() const;
	glm::mat4 getInverseModelMatrix() const;

	void addMesh(Mesh* mesh);
	AABB getAABB() const;
	void initAABB(glm::mat4 M);

	void update(float dt);
	
	void setParent(SceneObject* parent) { _parent = parent; }
	bool isEnabled() const { return _enabled; }
	void enable() { _enabled = true; }
	void disable() { _enabled = false; }
	std::string getName() const { return _name; }
	SceneObject* find(std::string name);

	void render(RenderInfo& renderinfo, const glm::mat4& Maccum, const glm::mat4& V, const glm::mat4& P) const;
};

