#include "Mesh.h"


Mesh::Mesh(std::initializer_list<VBO*> buffers, IBO* indicesBO, Material* material, size_t elementSize) 
	: _indicesBO(indicesBO), _material(material), _elementSize(elementSize)
{
	_vao = new VAO();
	_vao->setBuffers(_indicesBO, buffers);

	for (int i = 0; i < 9; i++) {
		_buffers[i] = nullptr;
	}

	typedef std::initializer_list<VBO*>::iterator iter;
	for (iter it = buffers.begin(); it != buffers.end(); it++) {
		_buffers[(*it)->getLocation()] = *it;
	}
}

Mesh::Mesh(std::vector<VBO*> buffers, IBO* indicesBO, Material* material, size_t elementSize)
	: _indicesBO(indicesBO), _material(material), _elementSize(elementSize)
{
	_vao = new VAO();
	_vao->setBuffers(_indicesBO, buffers);

	for (int i = 0; i < 9; i++) {
		_buffers[i] = nullptr;
	}

	typedef std::vector<VBO*>::iterator iter;
	for (iter it = buffers.begin(); it != buffers.end(); it++) {
		_buffers[(*it)->getLocation()] = *it;
	}
}

Mesh::~Mesh()
{
	for (int i = 0; i < 9; i++) {
		if (_buffers[i] != nullptr) delete _buffers[i];
	}
	delete _indicesBO;
	delete _vao;
}

void Mesh::setAABB(const AABB& aabb)
{
	_aabb = aabb;
}

AABB Mesh::getAABB() const
{
	return _aabb;
}

void Mesh::update(float dt)
{
	preupdate_sub(dt);

	_material->update(dt);

	postupdate_sub(dt);
}

void Mesh::render(RenderInfo& renderInfo, const glm::mat4& M, const glm::mat4& V, const glm::mat4& P) const
{
	Material* overwriteMaterial = renderInfo.overwriteMaterial;
	Material* currentMaterial = _material;
	if (overwriteMaterial != nullptr)
		currentMaterial = overwriteMaterial;

	_vao->bind();
	currentMaterial->setMeshDimensions(_aabb);
	
	currentMaterial->render(renderInfo, M, V, P);

	render_sub(renderInfo);
}