#pragma once

#include <glm.hpp>
#include <gtc/matrix_transform.hpp>

#include "../../../GLWrapper/VAO.h"
#include "../../../GLWrapper/VBO.h"
#include "../../../GLWrapper/IBO.h"
#include "../../material/Material.h"
#include "../../../render/RenderInfo.h"
#include "../AABB.h"


class Mesh
{
protected:
	VAO* _vao;
	VBO* _buffers[VBO_MAX_ATTRIBUTES]; // not more than VBO_MAX_ATTRIBUTES attributes 
	IBO* _indicesBO;
	size_t _elementSize;
	AABB _aabb;

	Material* _material;

	virtual void render_sub(RenderInfo& renderInfo) const = 0;
	virtual void preupdate_sub(float dt) = 0;
	virtual void postupdate_sub(float dt) = 0;

public:

	Mesh(std::initializer_list<VBO*> buffers, IBO* indicesBO, Material* material, size_t elementSize);
	Mesh(std::vector<VBO*> buffers, IBO* indicesBO, Material* material, size_t elementSize);
	virtual ~Mesh();

	void setAABB(const AABB& aabb);
	AABB getAABB() const; // world space
	
	void render(RenderInfo& renderInfo, const glm::mat4& M, const glm::mat4& V, const glm::mat4& P) const;

	void update(float dt);
};

