#include "ScreenQuadMesh.h"

static float _fullscreenVerts[] = {
	-1.0f, -1.0f, 0.0f, 
	1.0f, -1.0f, 0.0f,
	1.0f, 1.0f, 0.0f, 
	-1.0f, 1.0f, 0.0f 
};

static float _fullscreenUVs[] = {
	0.0f, 0.0f,
	1.0f, 0.0f,
	1.0f, 1.0f,
	0.0f, 1.0f
};

ScreenQuadMesh::ScreenQuadMesh(float width, float height, QUAD_CORNER corner, Material* material) :
	Mesh(initBuffers(width, height, corner), nullptr, material, 0)
{
}

ScreenQuadMesh::ScreenQuadMesh(Material* material) :
	Mesh(initBuffers(0, 0, QUAD_CORNER::FULL_SCREEN), nullptr, material, 0)
{
}

ScreenQuadMesh::~ScreenQuadMesh()
{
}

void ScreenQuadMesh::setMaterial(Material* material)
{
	this->_material = material;
}

std::vector<VBO*> ScreenQuadMesh::initBuffers(float width, float height, QUAD_CORNER quadCorner)
{
	float positions[] = {
		-1.0f, -1.0f, 0.0f, // bottom left
		1.0f, -1.0f, 0.0f, // bottom right
		1.0f, 1.0f, 0.0f, // top right
		-1.0f, 1.0f, 0.0f // top left
	};

	if (quadCorner != QUAD_CORNER::FULL_SCREEN) {
		float x = 0.0f;
		float y = 0.0f;

		switch (quadCorner)
		{
		case ScreenQuadMesh::QUAD_CORNER::TOP_LEFT:
			x = -1.0f + 2.0f * width;
			y = 1.0f - 2.0f * height;

			positions[3] = x;
			positions[4] = y;

			positions[1] = y;
			positions[6] = x;
			break;
		case ScreenQuadMesh::QUAD_CORNER::TOP_RIGHT:
			x = 1.0f - 2.0f * width;
			y = 1.0f - 2.0f * height;

			positions[0] = x;
			positions[1] = y;

			positions[9] = x;
			positions[4] = y;
			break;
		case ScreenQuadMesh::QUAD_CORNER::BOTTOM_LEFT:
			x = -1.0f + 2.0f * width;
			y = -1.0f + 2.0f * height;

			positions[6] = x;
			positions[7] = y;

			positions[10] = y;
			positions[3] = x;
			break;
		case ScreenQuadMesh::QUAD_CORNER::BOTTOM_RIGHT:
			x = 1.0f - 2.0f * width;
			y = -1.0f + 2.0f * height;
			positions[9] = x;
			positions[10] = y;

			positions[7] = y;
			positions[0] = x;
			break;
		}
	}

	return { new VBO(VBO::BUFFER_TYPE::POSITIONS, 4, positions),  new VBO(VBO::BUFFER_TYPE::UVS1, 4, _fullscreenUVs) };
}

void ScreenQuadMesh::render_sub(RenderInfo& renderInfo) const
{
	glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
}

void ScreenQuadMesh::preupdate_sub(float dt)
{

}

void ScreenQuadMesh::postupdate_sub(float dt)
{

}
