#include "Material.h"

#include "../graph/geometry/Mesh.h"

Material::Material(Shader* shader) : _shader(shader)
{
	_shininess = 1.0f;
	_ambientColor = glm::vec3(1.0f);
	_diffuseColor = glm::vec3(1.0f);
	_specularColor = glm::vec3(1.0f);
	_illuminationWeights = glm::vec3(0.2f, 0.7f, 0.1f);
}

Material::~Material()
{
}

void Material::setMaterialUniforms() const
{
	_shader->setUniform("matAmbColor", _ambientColor);
	_shader->setUniform("matDiffColor", _diffuseColor);
	_shader->setUniform("matSpecColor", _specularColor);
	_shader->setUniform("matShininess", _shininess);
	_shader->setUniform("matIlluminationWeights", _illuminationWeights);
}

void Material::setLightUniforms(std::vector<Light*> lights, LIGHT_TYPE type) const
{
	const char* uniformName = "directionalLights";
	const char* uniformNBName = "nbDirectionalLights";

	if (type == LIGHT_TYPE::POINT) {
		uniformName = "pointLights";
		uniformNBName = "nbPointLights";
	}

	_shader->setUniform(uniformNBName, (unsigned int)lights.size());

	for (unsigned int i = 0; i < lights.size(); i++) {
		if (lights[i]->enabled) {
			_shader->setUniformArr(uniformName, lights[i], i);
		}
	}
}

void Material::postDraw()
{
	_shader->unbind();
}
