#pragma once

#include <vector>
#include <glm.hpp>
#include <gtc/matrix_transform.hpp>
#include <gtc/matrix_inverse.hpp>

#include "../../GLWrapper/Shader.h"
#include "../../util/Global.h"
#include "../graph/AABB.h"

// forward declaration mark
struct RenderInfo;

class Material
{
protected:
	Shader* _shader;

	float _shininess;
	glm::vec3 _specularColor;
	glm::vec3 _diffuseColor;
	glm::vec3 _ambientColor;
	glm::vec3 _illuminationWeights;

	AABB _meshDimensions;

public:
	Material(Shader* shader);
	virtual ~Material();

	void setMaterialUniforms() const;
	void setLightUniforms(std::vector<Light*> lights, LIGHT_TYPE type) const;

	virtual void initShaderUniforms(const SceneGraph* scene) = 0;
	virtual void render(RenderInfo& renderInfo, const glm::mat4& M, const glm::mat4& V, const glm::mat4& P) const = 0;

	const float getShininess() { return _shininess; }
	const glm::vec3 getSpecularColor() { return _specularColor; }
	const glm::vec3 getDiffuseColor() { return _diffuseColor; }
	const glm::vec3 getAmbientColor() { return _ambientColor; }
	const glm::vec3 getIlluminationWeights() { return _illuminationWeights; }
	const AABB getMeshDimensions() { return _meshDimensions; }

	void setShininess(float shininess) { _shininess = shininess; }
	void setSpecularColor(glm::vec3 specularColor) { _specularColor = specularColor; };
	void setDiffuseColor(glm::vec3 diffuseColor) { _diffuseColor = diffuseColor; }
	void setAmbientColor(glm::vec3 ambientColor) { _ambientColor = ambientColor; }
	void setIlluminationWeights(glm::vec3 weights) { _illuminationWeights = weights; }
	void setMeshDimensions(AABB dimension) { _meshDimensions = dimension; }

	virtual void update(float dt) {}

	virtual void postDraw();
};

