#include "TexturePhongMaterial.h"

#include "../graph/SceneGraph.h"


TexturePhongMaterial::TexturePhongMaterial(Shader* shader)
	: TexturePhongMaterial(shader, nullptr)
{
}

TexturePhongMaterial::TexturePhongMaterial(Shader* shader, Texture2D* diffTex)
	: Material(shader), _diffTex(diffTex)
{
}

TexturePhongMaterial::~TexturePhongMaterial()
{
}

void TexturePhongMaterial::initShaderUniforms(const SceneGraph* scene)
{
	setMaterialUniforms();

	setLightUniforms(scene->getDirectionalLights(), LIGHT_TYPE::DIRECTIONAL);
	setLightUniforms(scene->getPointLights(), LIGHT_TYPE::POINT);
}

void TexturePhongMaterial::render(RenderInfo& renderInfo, const glm::mat4& M, const glm::mat4& V, const glm::mat4& P) const
{
	_shader->setUniform("MVP", P * V * M);
	_shader->setUniform("M", M);
	_shader->setUniform("V", V);
	_shader->setUniform("NormalM", glm::inverseTranspose(glm::mat3(M)));

	setLightUniforms(Global::instance().sceneGraph->getDirectionalLights(), LIGHT_TYPE::DIRECTIONAL);
	setLightUniforms(Global::instance().sceneGraph->getPointLights(), LIGHT_TYPE::POINT);

	if (_diffTex != nullptr) {
		_shader->setUniform("tex", _diffTex, 0);
	}

	_shader->bind();
}